//
//  SAAnimatedImageView.m
//
//  Created by Steffen Albrecht on 20.01.12.
//  Copyright (c) 2012 edsa edv dienstleistungen steffen albrecht. All rights reserved.
//

#import "SAAnimatedImageView.h"

@implementation SAAnimatedImageView
@synthesize delegate;

@synthesize frameCount = _frameCount;
@synthesize loopCount  = _loopCount;

-(void)showFrame {

    NSData* data = [dict valueForKey:[NSString stringWithFormat:@"image_%d", step]];
    IBASE* image = [[IBASE alloc] initWithData:data];
    [self setImage:image];
    [image release];
    
    [self setNeedsDisplay];
    
    if( playing )
        [self nextFrame];
}

-(void)nextFrame {
    
    step++;
    if( step >= self.frameCount ) {
        if( !playing || loop )
            step = 0;
        else {
            cycle++;
            step = self.frameCount-1;
            if( cycle>=self.loopCount ) {
                playing = NO;
                if( delegate!=nil )
                    [delegate imageViewDidFinish:self];
                return;
            }
        }
    }
    
    if( playing ) {
        float delay = [[dict valueForKey:[NSString stringWithFormat:@"duration_%d", step]] floatValue];
        [self performSelector:@selector(showFrame) withObject:nil afterDelay:delay];
    }
    else
        [self showFrame];
}

-(void)start:(BOOL)repeat {
    loop = repeat;

    //restart if already playing
    if( playing ) {
        step = 0;
        cycle = 0;
    }
    else 
    {
        if( !paused )
            step = 0;

        paused = NO;
        //start playing
        if( self.frameCount > 1 ) {
            playing = YES;
            float delay = [[dict valueForKey:[NSString stringWithFormat:@"duration_%d", step]] floatValue];
            [self performSelector:@selector(showFrame) withObject:nil afterDelay:delay];
        }
    }
}

-(void)reset {
    step = 0;
    cycle = 0;
    paused = NO;
    playing = NO;
    [self showFrame];
}

-(void)stop {
    step = 0;
    cycle = 0;
    paused = NO;
    playing = NO;
}

-(void)pause {
    if( paused ) {
        paused = NO;
        //start playing
        if( self.frameCount > 1 ) {
            playing = YES;
            float delay = [[dict valueForKey:[NSString stringWithFormat:@"duration_%d", step]] floatValue];
            [self performSelector:@selector(showFrame) withObject:nil afterDelay:delay];
        }
    }
    else {
        playing = NO;
        paused = YES;
    }
}

-(void)setDictFilename:(NSString*)filename fromBundle:(BOOL)bundle {
    
    if( dict != nil )
        [dict release];
    
    if( bundle )
        filename = [[NSBundle mainBundle] pathForResource:filename ofType:@""];
    dict = [[NSDictionary alloc] initWithContentsOfFile:filename];
    
    self.frameCount = [[dict valueForKey:@"framecount"] integerValue];
    self.loopCount  = [[dict valueForKey:@"loopcount"] integerValue];
    
    step = 0;
    paused = NO;
    playing = NO;
    
    [self showFrame];
}

-(void)dealloc {
    [dict release];
    [super dealloc];
}

@end